SortedCollection variableSubclass: #Calendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
Calendar comment:
'  Tato trida implementuje prostredek pro koordinaci vyskytu udalosti v modelovanem systemu vzhledem k simulacnimu casu nazyvany ''planovaci kalendar'' (Kalendar udalosti). Jeho polozkami jsou tedy entity, tj. instance tridy ''SimProcess'' a jejich podtrid. 
  Dedi tridu ''SortedCollection'' (usporadany seznam polozek podle zadaneho klice).
  Instancni ani tridni promenne nema.'!


!Calendar methodsFor: 'removing'!

clear
	"Zrusi vsechny polozky (entity) z kalendare 
	a vsechny je pasivuje (aktivacni cas = -1). 
	Soucasne nastavi cas pristiho vyskytu udalosti 
	v aktivnim modelu za konec mozneho casoveho intervalu"

	MySimulation active nextTime: 1.0e38.
	[self isEmpty]
		whileFalse: 
			[self last activationTime: -1.
			self removeLast]!

get: entity 
	"Zrusi polozku 'entity' v kalendari udalosti a vraci odkaz na ni. 
	Neni-li polozka nalezena, dochazi k chybe. Zaroven nastavi 
	v modelu pristi hodnotu simulacniho casu."

	| found |
	self isEmpty ifTrue: [MySimulation active error: 'Pokus o zruseni entity v prazdnem kalendari'].
	found := self remove: entity ifAbsent: [MySimulation active error: 'Entita neni naplanovana'].
	found activationTime: -1.
	self isEmpty
		ifTrue: [MySimulation active nextTime: 1.0e38]
		ifFalse: [MySimulation active nextTime: self first activationTime].
	^found! !

!Calendar methodsFor: 'accessing'!

current
	"Zpristupni prvni polozku seznamu (vraci na ni odkaz). 
	Je-li kalendar prazdny, dochazi k chybe."

	self isEmpty ifTrue: [MySimulation active error: 'Prazdny kalendar - nelze zpristupnit                                          prvni polozku'].
	^self first! !

!Calendar methodsFor: 'adding'!

schedule: entity at: aTime 
	"Pozadavek na vlozeni polozky 'entity' s aktivacnim casem 'aTime' 
	do kalendare (tj. planovani nasledujici udalosti v teto entite). 
	Pripadne predchozi planovani entity je zruseno. Je-li polozka 
	planovana na cas < aktualni simulacni cas, dochazi k chybe."

	MySimulation time > aTime ifTrue: [MySimulation active error: 'Planovani pred aktualni hodnotou simulacniho casu'].
	entity activationTime: aTime.
	self remove: entity ifAbsent: [].
	self add: entity.
	MySimulation active nextTime: self first activationTime! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Calendar class
	instanceVariableNames: ''!


!Calendar class methodsFor: 'instance creation'!

new
	"Vytvoreni kalendare udalosti pomoci metody
      supertridy 'SortedCollection'. 
	Usporadani polozek v seznamu (entity) je dano: 
	 1. casem jejich aktivace (vzestupne) 
	 2. prioritou (sestupne) 
	 3. (implicitne) strategii FIFO"

	^super sortBlock: [:x :y | x activationTime <= y activationTime & (x priority >= y priority)]! !

Object subclass: #Entity
	instanceVariableNames: 'waitingQueue actTime inpTime priority sPriority rCapacity '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
Entity comment:
'Trida ''Entity'' je abstraktni bazova trida definujici zakladni atributy entity (krome jejiho chovani) a metody pro rizeni jejiho stavu. Stav je zakladnim atributem kazde entity - entita muze byt aktivni nebo pasivni (naplanovana nebo nenaplanovana v kalendari udalosti). Tato trida tedy definuje instancni metody pro rizeni stavu entity (vzhledem ke kalendari udalosti), metody pro styk entity s objekty typu fronta, sklad a zarizeni a metody pro praci s jejimi atributy.

Promenne instanci:
  actTime <Float>	- aktivacni cas entity (neni-li entita naplanovana, je actTime roven -1)
  priority <Integer>	- priorita entity (v systemu)
  sPriority <Integer>	- priorita obsluhy entity (uplatnovana pri obsazovani zarizeni)
  waitingQueue <Queue> - odkaz na frontu, do niz byla entita umistena (implicitne nil)
  inpTime <Float>	- cas vstupu entity do fronty ''waitingQueue''
  rCapacity	 <Integer>- pozadovana kapacita prostredku typu sklad, kterou entita obsazuje(-dila)'!


!Entity methodsFor: 'state control'!

activate
	"Planovani aktivace entity na aktualni hodnotu simulacniho casu."

	self activateAt: MySimulation time!

activate: aTime
	"Planovani aktivace entity na cas 'aTime'."

	self activateAt: aTime!

activateAt: aTime
	"Planovani aktivace entity na cas 'aTime',
	zajistuje primo planovani entity do kalendare udalosti."

	MySimulation active calendar schedule: self at: aTime!

passivate
	"Zrusi pripadne planovani entity (uvede entitu do pasivniho stavu)."

	self idle ifFalse: [MySimulation active calendar get: self]!

wait: wTime
	"Planuje reaktivaci entity po uplynuti doby 'wTime',
	tj. v case 'aktualni cas + wTime'"

	self activateAt: MySimulation time + wTime! !

!Entity methodsFor: 'private'!

initVariables
	"Inicializace atributu (instancnich promennych) entity."

	priority := 0.
	sPriority := 0.
	rCapacity := 0.
	actTime := -1.
	waitingQueue := nil.
	inpTime := -1! !

!Entity methodsFor: 'queueing'!

into: queue 
	"Zajisti vlozeni entity do fronty 'queue'; 
	nastavi hodnotu promenne pro odkaz na tuto frontu."

	queue add: self.
	waitingQueue := queue!

out
	"Vyradi entitu z fronty, do niz byla umistena (odkaz na ni 
	je ulozen v inst. promenne 'waitingQueue')."

	waitingQueue isNil ifFalse: [waitingQueue remove: self]
		ifTrue: [MySimulation active error: 'Entitu nelze vyradit z fronty (spatny odkaz na frontu)']!

waitQ: queue 
	"Zajisti vstup entity do fronty queue. 
	Je-li entita aktivni, pasivuje ji."

	self into: queue.
	self passivate! !

!Entity methodsFor: 'accessing'!

activationTime
	"Zpristupni hodnotu aktivacniho casu entity (aktivacni cas entity, 
	ktera neni planovana ke spusteni, je roven -1)."

	^actTime!

activationTime: aTime 
	"Nastavi aktivacni cas entity na dobu 'aTime'."

	actTime := aTime!

idle
	"Je-li planovana reaktivace entity, 
	vraci booleovskou promennou 'true'."

	^actTime = -1!

inpTime
	"Vraci cas vstupu entity do fronty, v niz je entita umistena."

	^inpTime!

inpTime: aTime 
	"Nastavi cas vstupu entity do fronty na hodnotu 'aTime'."

	inpTime := aTime!

priority
	"Vraci prioritu entity (priorita v simulovanem systemu)."

	^priority!

priority: newPriority 
	"Nastavi prioritu entity na hodnotu 'newPriority'."

	priority := newPriority!

reqCap
	"Zpristupni velikost kapacity skladu obsazovaneho entitou."

	^rCapacity!

reqCap: newCapacity 
	"Nastavi novou pozadovanou kapacitu skladu obsazovaneho entitou."

	rCapacity := newCapacity!

sPriority
	"Zpristupni prioritu obsluhy entity."

	^sPriority!

sPriority: newSPriority 
	"Nastavi novou prioritu obsluhy entity."

	sPriority := newSPriority!

waitingQueue
	"Zpristupni odkaz na frontu, v niz entita je entita umistena."

	^waitingQueue!

waitingQueue: queue 
	"Ulozi odkaz na frontu 'queue' do instancni promenne entity."

	waitingQueue := queue! !

Entity subclass: #SimProcess
	instanceVariableNames: 'behavior running '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
SimProcess comment:
'Bazova trida pro vytvareni novych trid entit v systemu (spolu s tridou ''Entity'' vytvari koren hierarchie techto trid). Chovani entity je reprezentovano smalltalkovskym procesem, jehoz jadrem je blok popisujici chovani entity posloupnosti vyrazu. Aktivace ci pasivace entity se jednoduse provadi zmenou stavu tohoto procesu (zasilanim zprav ''resume'', ''suspend'' ci ''terminate''). Instancni metody tvori nadstavbu metod supertridy, ktere doplnuji o prostredky pro realizaci chovani entity. Pred vstupem do simulace musi mit entita sve chovani (odkazovane instancni promennou) definovano, jinak dochazi k chybe.

Instancni promenne
behavior <Process>	- predstavuje proces reprezentujici chovani entity.
                         Chovani je definovane posloupnosti vyrazu (predstavujicich udalosti), 
                         ktere jsou umisteny v bloku vyhodnocovanem v ramci tohoto procesu.
 running <Boolean>	- vymezuje oblast chovani entity mezi jeji aktivaci a pasivaci, tj. pro
                          spusteny proces, ktery nebyl prerusen udalosti v chovani entity,
                          jez reprezentuje, ma tato promenna hodnotu ''true''.
                          '!


!SimProcess methodsFor: 'initialization'!

initialize: aBlock 
	"Inicializuje chovani entity, ktere je urceno argumentem 'aBlock' 
	(tj. vytvori novy proces, jez bude toto chovani realizovat a odkaz
	na nej ulozi do prislusne instancni promenne 'behavior' ."

	MySimulation active addEntity.
	running := false.
	behavior := 
			[aBlock value.
			running := false.
			MySimulation active subEntity.
			super passivate] newProcess! !

!SimProcess methodsFor: 'task language'!

activate
	"Planuje aktivaci entity na aktualni simulacni cas; vyuziva metodu 
	supertridy pro planovani entity do kalendare udalosti. Soucasne 
	prerusi provadeni jejiho chovani."

	super activate.
	self interrupt!

activate: aTime 
	"Planuje aktivaci entity na cas 'aTime', tj. vlozi entitu do kalendare 
	udalosti a pasivuje ji."

	super activate: aTime.
	self interrupt!

enter: store 
	"Pozadavek entity na obsazeni jednoho mista ve skladu 'store'."

	store enter: self capacity: 1!

enter: store capacity: reqcap 
	"Pozadavek na obsazeni kapacity 'reqcap' ve skladu 'store'."

	store enter: self capacity: reqcap!

leave: store 
	"Uvolneni kapacity skladu 'store' obsazene entitou."

	store leave: rCapacity!

passivate
	"Pozastavi proces realizujici chovani entity a soucasne zrusi pripadne 
	planovani jeji reaktivace (aktivita entity muze byt tedy obnovena 
	jen prostrednictvim jine entity)."

	super passivate.
	self interrupt!

release: facility 
	"Zadost o uvolneni zarizeni 'facility' touto entitou."

	facility release: self!

seize: facility 
	"Pozadavek na obsazeni zarizeni 'facility' s prioritou obsluhy 0."

	facility seize: self priority: 0!

seize: facility priority: prior 
	"Pozadavek na obsazeni zarizeni 'facility' s prioritou obsluhy 'prior'."

	facility seize: self priority: prior!

terminate
	"Ukonci a zrusi proces realizujici chovani entity, tj. ukonci jeji 
	existenci v procesu simulace."

	self terminateProcess!

wait: wTime 
	"Pozastavi proces realizujici chovani entity a naplanuje jeji reaktivaci 
	po uplynuti doby 'wTime' (s vyuzitim metody supertridy)."

	super wait: wTime.
	self interrupt! !

!SimProcess methodsFor: 'entity control'!

interrupt
	"Prerusi provadeni chovani entity; neni-li chovani definovano, dochazi 
	k chybe."

	behavior isNil ifTrue: [MySimulation active error: 'Pokus o zastaveni entity, jejiz chovani nebylo inicializovano.'].
	running ifTrue: [self suspendProcess]!

run
	"Spusti proces realizujici chovani entity v prubehu simulace (pri 
	pokusu o spusteni neinicializovane entity dochazi k chybe)."

	behavior isNil ifTrue: [MySimulation active error: 'Spusteni entity, jejiz chovani nebylo inicializovano'].
	self resumeProcess! !

!SimProcess methodsFor: 'private'!

resumeProcess
	"Spusti proces realizujici chovani entity."

	running := true.
	behavior resume!

suspendProcess
	"Zastavi bezici proces realizujici chovani entity."

	running := false.
	behavior suspend!

terminateProcess
	"Zrusi planovani entity a provede vsechny akce vedouci k ukonceni 
	existence entity v systemu."

	behavior isNil ifTrue: [MySimulation active error: 'Pokus o zruseni entity, jejiz chovani nebylo inicializovano.'].
	running := false.
	MySimulation active subEntity.
	super passivate.
	behavior terminate! !

!SimProcess methodsFor: 'accessing'!

inSimulation
	"Poskytuje informaci o stavu entity - neni-li cinnost entity jeste 
	ukoncena, pak vraci 'true'."

	^behavior suspendingList ~= nil | running!

isRunning
	"Poskytuje informaci o stavu procesu realizujiciho chovani entity. Je-li 
	tento proces aktivni (jeho cinnost nebyla pozastavena nekterou z
	udalosti v provadenem chovani), vraci hodnotu 'true'."

	^running! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimProcess class
	instanceVariableNames: ''!


!SimProcess class methodsFor: 'instance creation'!

new
	"Vytvari novou entitu s nedefinovanym chovanim a inicializuje jeji 
	atributy (definovane v supertride 'Entity')."

	^super new initVariables!

new: aBlock 
	"Vytvari novou entitu s chovanim definovan7m argumentem 'aBlock' a 
	inicializuje vsechny jeji atributy."

	^(super new) initVariables; initialize: aBlock! !

Object subclass: #Stat
	instanceVariableNames: 'sumX min max num sName '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
Stat comment:
'Trida Stat definuje charakteristiky a akce objektu, ktere jsou v simulacnim systemu urceny ke sberu hodnot a jejich zpracovani. Poskytuje udaje typu:
- maximalni a minimalni hodnota vstupu
- pocet vstupnich hodnot
- soucet vstupnich hodnot
Z jejich udaju lze urcit prumernou hodnotu vstupu.

Promenne  instance: 
  num <Integer> - pocet vstupnich hodnot
  sumX <Float> 	- soucet vstup. hodnot
  max <Float> 	- maximalni vstupni hodnota
  min <Float> 	- minimalni vstupni hodnota
  sName <String> - oznaceni statistiky'!


!Stat methodsFor: 'initialization'!

clear
	"Uvede vsechny atributy statistiky do pocatecniho stavu."

	self initialize: ''!

initialize: name 
	"Inicializuje vsechny atributy statistiky."

	sName := name.
	sumX := 0.
	min := 0.
	max := 0.
	num := 0! !

!Stat methodsFor: 'accessing'!

getName
	"Vraci jmeno statistiky."

	^sName!

max
	"Vraci maximum ze zaznamenanych vstupnich hodnot."

	^max!

min
	"Vraci minimum ze zaznamenanych vstupnich hodnot."

	^min!

num
	"Vraci pocet zaznamenanych hodnot."

	^num!

setName: newName 
	"Nastavi nove jmeno statistiky."

	sName := newName!

sumX
	"Vraci soucet zaznamenanych hodnot."

	^sumX! !

!Stat methodsFor: 'recording'!

rec: x 
	"Zaznamena vstupni hodnotu x do statistiky, tj. aktualizuje hodnoty 
	jejich atributu."

	sumX := sumX + x.
	num := num + 1.
	num = 1
		ifTrue: 
			[min := x.
			max := x]
		ifFalse: 
			[x < min ifTrue: [min := x].
			x > max ifTrue: [max := x]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Stat class
	instanceVariableNames: ''!


!Stat class methodsFor: 'instance creation'!

new
	"Vytvari novou statistiku a inicializuje jeji atributy."

	^super new initialize: ''!

new: sName 
	"Vytvari novou statistiku s nazvem 'sName' a inicializuje jeji atributy."

	^super new initialize: sName! !

Object subclass: #Histogram
	instanceVariableNames: 'hName valArray low high step stat '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
Histogram comment:
'Trida Histogram definuje objekty urcene pro zaznam sledovanych hodnot do zadanych intervalu. Z nich lze vypocitat cetnost hodnot v techto intervalech a pripadne ji graficky znazornit. K uchovani udaju o vstupnich hodnotach vyuziva histogram tyto promenne:

Promenne instance:
 low <Float> 	- dolni mezni hodnota histogramu
 high <Float> 	- horni mezni hodnota histogramu
 step <Float> 	- sirka jednoho intervalu z celeho rozsahu hodnot histogramu
 hName <String> - jmeno histogramu
 valArray <Array> - pole zaznamenavanych hodnot (polozka odpovida jednomu intervalu a jeji
      obsah poctu vstupnich hodnot, ktere do tohoto intervalu patrily
  stat <Stat>	- statistika histogramu pro uchovani dalsich udaju o vstupnich hodnotach'!


!Histogram methodsFor: 'accessing'!

at: i 
	"Zpristupni i-ty interval pole vstupnich hodnot (vraci pocet zaznamu 
	v tomto intervalu). Jestlize 'i' lezi mimo rozsah pole, dochazi k chybe."

	low + (i * step) > high
		ifTrue: [^valArray at: low + (i * step)]
		ifFalse: [MySimulation active error: 'Chyba pri pristupu k neexistujicimu prvku u histogramu ' , hName]!

getName
	"Vraci nazev histogramu."

	^hName!

lower
	"Vraci dolni mez rozsahu hodnot histogramu."

	^low!

setName: newName 
	"Nastavi novy nazev histogramu."

	hName := newName!

stat
	"Zpristupni statistiku histogramu."

	^stat!

step
	"Vraci krok zaznamu, tj. velikost jednoho intervalu z rozsahu 
	vstupnich hodnot (krok histogramu)."

	^step!

upper
	"Vraci horni mez rozsahu hodnot histogramu."

	^high! !

!Histogram methodsFor: 'printing'!

firstHeader: aStream 
	"Pomocna metoda pro vystup histogramu - vypisuje prvni textovou 
	hlavicku."

	aStream cr; tab.
	aStream nextPutAll: '  Pocet  '.
	aStream tab.
	aStream nextPutAll: '      Minimalni '.
	aStream tab.
	aStream nextPutAll: '      Maximalni '.
	aStream tab.
	aStream nextPutAll: '           Prumerna '.
	aStream cr; tab.
	aStream nextPutAll: ' objektu: '.
	aStream tab.
	aStream nextPutAll: '       hodnota: '.
	aStream tab.
	aStream nextPutAll: '       hodnota: '.
	aStream tab.
	aStream nextPutAll: '            hodnota: '!

output: aStream 
	"Realizuje vypis histogramu na textovy vystup specifikovany 
	parametrem 'aStream', tj. vypise jednotlive intervaly s poctem 
	zaznamenanych hodnot a cetnosti prislusejici temto intervalum, ktere 
	zobrazi delkou sloupce znaku '+'."

	| pos max div |
	stat num = 0
		ifTrue: 
			[aStream cr; nextPutAll: 'Histogram `' , hName , '` neobsahuje zadny zaznam.'; cr.
			^nil].
	max := valArray at: 1.
	valArray do: [:x | x > max ifTrue: [max := x]].
	div := max // 35 + 1.	"aby se to vlezlo na radku"
	aStream cr; cr; nextPutAll: 'Histogram - ' , hName; cr.
	aStream nextPutAll: '---------'.
	self firstHeader: aStream.
	aStream cr; tab.
	self
		print: stat num
		with: 10
		on: aStream.
	aStream tab; tab.
	self
		print: stat min
		with: 10
		on: aStream.
	aStream tab; tab.
	self
		print: stat max
		with: 10
		on: aStream.
	aStream tab; tab.
	stat num = 0
		ifTrue: [self
				print: 0.0
				with: 15
				on: aStream]
		ifFalse: [self
				print: (stat sumX / stat num) asFloat
				with: 15
				on: aStream].
	aStream cr.
	self secondHeader: aStream.
	aStream cr.
	pos := low.
	valArray do: 
		[:entry | 
		pos = low
			ifTrue: [aStream nextPutAll: '     <- ']
			ifFalse: [self
					print: pos - step
					with: 8
					on: aStream].
		aStream nextPutAll: ' -'.
		pos = (valArray size - 1 * step + low)
			ifTrue: [aStream nextPutAll: '     -> ']
			ifFalse: 
				[pos := pos + step.
				self
					print: pos - step
					with: 8
					on: aStream].
		aStream tab.
		self
			print: entry
			with: 6
			on: aStream.
		aStream tab.
		stat num = 0
			ifTrue: [self
					print: 0.0
					with: 14
					on: Stream]
			ifFalse: [self
					print: (entry / stat num) asFloat
					with: 14
					on: aStream].
		aStream tab.
		aStream nextPut: $|.
		(entry / div) rounded timesRepeat: [aStream nextPut: $+].
		aStream cr]!

print: number with: spacing on: aStream 
	"Pomocna metoda vypisujici cislo 'number' na pocet mist specifikovany 
	argumentem 'spacing' na vystup 'aStream'."

	| myStream |
	myStream := WriteStream on: (String new: 10).
	number printOn: myStream.
	myStream position < spacing ifTrue: [spacing - myStream position timesRepeat: [aStream nextPut: $ ]].
	number printOn: aStream!

secondHeader: aStream 
	"Pomocna metoda pro vystup histogramu - vypisuje druhou textovou 
	hlavicku."

	aStream cr; tab.
	aStream nextPutAll: '                     Pocet '.
	aStream cr.
	aStream nextPutAll: '        Interval '.
	aStream tab; tab.
	aStream nextPutAll: ' objektu '.
	aStream tab.
	aStream nextPutAll: '   Frekvence '! !

!Histogram methodsFor: 'recording'!

store: val 
	"Zaznam nove hodnoty 'val' do histogramu, tj. aktualizace jeho pole 
	vstupnich hodnot a statistiky."

	| index |
	stat rec: val.
	val < low
		ifTrue: [index := 1]
		ifFalse: [val >= (valArray size - 1 * step + low)
				ifTrue: [index := valArray size]
				ifFalse: [index := (val - low / step) truncated + 2]].
	valArray at: index put: (valArray at: index)
			+ 1! !

!Histogram methodsFor: 'initialization'!

clear
	"Uvede vsechny atributy histogramu do pocatecniho stavu."

	valArray atAllPut: 0.
	stat clear!

initialize: hname from: lower to: upper by: stepH 
	"Inicializuje atributy histogramu, tj. rozsah vstupnich hodnot dany 
	intervalem <lower, upper>, krok zaznamu <stepH>, jmeno <hname>, 
	pole vstupnich hodnot a statistiku histogramu."

	| pom |
	hName := hname.
	stepH <= 0 ifTrue: [MySimulation active error: 'Chybne zadany krok v histogramu ' , hName].
	upper <= lower ifTrue: [MySimulation active error: 'Chybne zadane meze u histogramu ' , hName].
	pom := upper - lower / stepH asFloat.
	pom fractionPart > 0 ifTrue: [pom := pom + 1].
	valArray := Array new: pom truncated + 2.
	valArray atAllPut: 0.
	low := lower.
	high := upper.
	step := stepH.
	stat := Stat new: 'Statistika histogramu ' , hName! !

!Histogram methodsFor: 'user interface'!

inspect
	"Otevre okno inspektoru pro histogram. Inspektor umoznuje prohlizeni 
	jeho atributu a vypis histogramu na systemovy vystup (System 
	Transcript)."

	SimInspectorView open: (HistogramInspector inspect: self)
		label: 'Histogram `' , hName , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Histogram class
	instanceVariableNames: ''!


!Histogram class methodsFor: 'instance creation'!

from: lower to: upper by: stepH 
	"Vytvori novy histogram a aktivuje metodu pro inicializaci jeho 
	atributu."

	^self new
		initialize: ''
		from: lower
		to: upper
		by: stepH!

new: hname from: lower to: upper by: stepH 
	"Vytvori novy histogram s nazvem 'hname' a aktivuje metodu pro 
	inicializaci jeho atributu."

	^self new
		initialize: hname
		from: lower
		to: upper
		by: stepH! !

Object subclass: #MySimulation
	instanceVariableNames: 'myCalendar phase startTime endTime stopFlag simTime nextTime count actCount integrator status condition graph stepNum resetStatus optStep sizeStep minStep maxStep absError relError contractStepFlag contractStep dynamicFlag conditionFlag method stepStartTime '
	classVariableNames: 'ActiveSimulation SuspendedQueue '
	poolDictionaries: ''
	category: 'BasicSimulation'!
MySimulation comment:
'Trida ''MySimulation'' je bazovou tridou vsech simulacnich modelu (kategorie ''SimExamples''). Jejimi instancemi (a instancemi jejich subtrid) jsou tedy konkretni modely obsahujici vsechny prostredky k realizaci simulacnich behu. Tyto modely maji k dispozici vlastni kalendar udalosti a promennou predstavujici simulacni cas. Kazdy simulacni beh je rozdelen do techto fazi:
- pocatecni faze (start) - po vzniku modelu
- inicializacni faze (initialization) - inicializace vsech parametru simulace
- vlastni simulace (simulation) - hodnoty simulacniho casu jsou rizeny vyskytem pristi udalosti (tj. casem aktivace prislusne entity) naplanovane v kalendari udalosti
- ukonceni simulace (termination) - simulacni cas dosahl sve koncove hodnoty nebo byla simulace predcasne ukoncena.
Pred inicializaci experimentu se vytvareji a inicializuji objekty v simulacnim modelu (vetsinou ve forme instancnich promennych); vytvoreni a naplanovani entit se provadi az po nastaveni meznich hodnot simulacniho casu.
Samotna trida umoznuje ridit kvaziparalelni provadeni vice simulaci soucasne, ktere musi byt spusteny na pozadi. V kazdem okamziku je aktivni pouze jeden simulacni model, na nejz odkazuje promenna tridy MySimulation - ActiveSimulation. Pres tuto promennou ziskavaji objekty aktivniho modelu odkaz na tento model. Vsechny ostatni simulace jsou pozastaveny a umisteny ve fronte pozastavenych procesu ve tride MySimulation. Polozky teto fronty tvori dva odkazy - odkaz na simulovany model a odkaz na proces, ktery jej reprezentuje. Predavani rizeni dalsimu modelu zajistuji metody tridy, ktere jsou aktivovany pri zmene simulacniho casu aktivniho modelu (aktivni model je pozastaven a vlozen na konec fronty a model z cela fronty je spusten). Je-li nektery z modelu spusten primo, tj. ne ve forme procesu na pozadi, je prednostne provaden - ostatni modely ve fronte musi cekat na ukonceni jeho simulacniho behu.
Nove simulacni modely se musi ''hlasit'' u tridy MySimulation, tj. musi byt vlozeny ve forme nove polozky do fronty pozastavenych simulaci a musi jim byt predano rizeni - tj. je nutne je aktivovat. Behem aktivace je pozastaven simulovany model (pripadne zrusen, neni-li spusten na pozadi).

Promenne tridy:
  ActiveSimulation <MySimulation> - odkaz na aktivni simulacni model
  SuspendedQueue <OrderedCollection> - fronta pozastavenych simulaci, jejimi polozkami jsou pole (Array) o velikosti dvou prvku (odkaz na proces a simulovany model)

Promenne instanci:
  myCalendar <Calendar>	- kalendar udalosti simulovaneho modelu
  phase <Symbol> 			- aktualni faze simulacniho behu
  startTime <Double>			- pocatecni hodnota sim. casu
  endTime <Double>			- koncova hodnota sim. casu
  simTime <Double>			- aktualni  -   ||   -
  nextTime <Double>  		- nasledujici  -  ||  -
  stopFlag <Boolean>		- priznak ukonceni simulace
  count <Integer>			- pocet vsech entit, ktere vznikly od zacatku simulace
  actCount <Integer>		- aktualni pocet entit v simulacnim systemu
  integrator <Array>            - pole odkazu na integratory
  status <Array>            - pole odkazu na stavove bloky
  condition <Array>            - pole odkazu na podminky
  graph <Array>            - pole odkazu na grafy
  stepNum <Float>          - pocet kroku spojite simulace
  resetStatus <Boolean>    - priznak nutnosti inicializovat krok spojite simulace
  optStep <Double>         - optimalni krok spojite simulace
  sizeStep <Double>        - krok spojite simulace
  minStep <Double>         - minimalni krok
  maxStep <Double>         - maximalni krok
  absError <Double>        - absolutni chyba numericke integrace
  relError <Double>        - relativni chyba numericke integrace
  contractStepFlag <Boolean> - priznak zkracovani kroku
  contractStep <Double>      - mozny budouci krok
  dynamicFlag <Boolean>      - priznak dynamicke sekce
  conditionFlag <Boolean>    - priznak zmeny nektere podminky
  method <String>            - jaka metoda se bude provadet
  stepStartTime <Double>     - cas zacatku kroku spojite simulace'!


!MySimulation methodsFor: 'initialization'!

initialize
	"Inicializace charakteristik simulacniho modelu a generovani
	nahodnych cisel."

	count := 0.
	actCount := 0.
	phase := #START.
	nextTime := 1.0e38 asDouble.
	simTime := 0 asDouble.
	stopFlag := false.
	integrator := Array new: 0.
	status := Array new: 0.
	condition := Array new: 0.
	graph := Array new: 0.
	myCalendar := Calendar new.
	BasicRandom initialize! !

!MySimulation methodsFor: 'private'!

activate
	"Zajistuje aktivaci modelu, tj. nastaveni odkazu ve tride MySimulation na tento model.
     Pripadna simulace probihajici na pozadi je pozastavena. Neni-li predchozi simulace
     spustena na pozadi, uzivatel sam rozhodne o jejim zastaveni ci nezastaveni. Pokud
     model nelze aktivovat, je o tom uzivatel informovan."

	| query |    "query = true znamena, ze simulace neprobiha na pozadi a uzivatel musi
                    rozhodnout o jejim zastaveni."
	query := false.
	ActiveSimulation == self ifTrue: [^true].  "Uz jsem byl aktivovan"
	ActiveSimulation isNil
		ifTrue: [
			ActiveSimulation := self]
		ifFalse: [SuspendedQueue isEmpty
				ifTrue: [query := true]
				ifFalse: [(SuspendedQueue first at: 2)
						== ActiveSimulation ifFalse: [query := true]
						ifTrue:
							[(SuspendedQueue first at: 1) suspend.
							[(SuspendedQueue first at: 2) calendar current inSimulation]
								whileTrue: [Processor yield].
							ActiveSimulation := self]]].
	query ifTrue: [(self confirm: 'Ukoncit simulaci modelu `' , ActiveSimulation class name , '` ?')
			ifTrue:
				[ActiveSimulation stop.
				ActiveSimulation := self]
			ifFalse: [ActiveSimulation isNil ifFalse: [self error: 'Nemohu aktivovat tento sim. model']]]!

addEntity
	"Inkrementuje citace entit pri vzniku nove entity."

	actCount := actCount + 1.
	count := count + 1!

subEntity
	"Dekrementuje aktualni citac entit pri ukonceni existence entity v
	systemu."

	actCount := actCount - 1! !

!MySimulation methodsFor: 'simulation control'!

init: bTime to: eTime 
	"Provadi inicializacni fazi simulacniho experimentu - nastavi krajni 
	hodnoty intervalu simulacniho casu (simulacni cas bude probihat 
	intervalem <bTime,eTime>) a inicializuje kalendar udalosti. Osetruje 
	i pripadne chyby, ktere mohou nastat behem inicializacni faze."

	phase == #INITIALIZATION ifTrue: [self error: 'Pokus o opakovanou inicializaci simulace.'].
	phase == #SIMULATION ifTrue: [self error: 'Pokus o inicializaci v prubehu simulace.'].
	phase := #INITIALIZATION.
	bTime >= eTime | (bTime < 0) | (eTime > 1.0e38) ifTrue: [self error: 'Chybne parametry pri inicializaci simulace.'].
	startTime := bTime asDouble.
	simTime := startTime asDouble.
	endTime := eTime asDouble.
	myCalendar clear!

run
	"Zajistuje vlastni prubeh simulace - ridi postupnou aktivaci entit 
	podle kalendare udalosti a odpovidajici zmeny simulacniho casu. Pred 
	kazdou zmenou sim. casu je zaslana zprava tride MySimulation, ktera 
	ridi aktivaci dalsiho modelu nebo ponecha aktivni tento model. 
	Ukonceni simulace nastava pote, co sim. cas dosahne sve koncove 
	hodnoty, kalendar udalosti je uz prazdny nebo byl nastaven priznak 
	ukonceni simulace. O ukonceni simulace je informovana trida 
	MySimulation, aby pripadne vyradila tento model z dalsiho rizeni."

	| nTime hours min sec milisec |
	nTime := 0.
	phase ~= #INITIALIZATION ifTrue: [self error: 'Nevhodne pouziti metody run (chybi inicializace systemu)'].
	nextTime < startTime ifTrue: [self error: 'Chyba pri spousteni simulace.'].
	phase := #SIMULATION.
	stepNum := 0.
	self continueInit.
	self graphInit.
	[simTime < endTime & stopFlag not]
		whileTrue: 
			[stopFlag := endTime < nextTime.
			nextTime > endTime ifTrue: [nextTime := endTime].
			nextTime > simTime ifTrue: ["spojita simulace"
				self isAny
					ifTrue: 
						[resetStatus := true.
						[simTime < nextTime]
							whileTrue: 
								[nTime := nTime + (Time millisecondsToRun: [self stepSimulation]).
								stepNum := stepNum + 1.
								self doCondition]]
					ifFalse: [simTime := nextTime]].
			[simTime = nextTime & myCalendar isEmpty not & stopFlag not]
				whileTrue: 
					[| currentE |
					currentE := myCalendar current.
					currentE run.
					[currentE isRunning]
						whileTrue: ["cekani na preruseni chovani entity"
							"predavani procesoru dalsim procesum 
							umoznuje spusteni simulace na pozadi !!"
							Processor yield]].
			MySimulation suspendActive].
	MySimulation cancelActive.
	milisec := ((nTime / 1000) asDouble fractionPart * 1000) truncated.
	nTime := nTime - milisec / 1000.
	sec := (nTime \\ 60) truncated.
	min := nTime // 60.
	hours := (min // 60) truncated.
	min := (min \\ 60) truncated.
	Transcript show: 'Cas vypoctu:  '; print: hours; nextPutAll: ':'; print: min; nextPutAll: ':'; print: sec; nextPutAll: ','; print: milisec; cr; endEntry.
	phase := #TERMINATION!

stop
	"Zadost o predcasne ukonceni simulacniho behu nastavenim priznaku
	ukonceni."

	phase == #SIMULATION
		ifTrue: 
			[stopFlag := true.
			[phase ~= #TERMINATION]
				whileTrue: [Processor yield]]! !

!MySimulation methodsFor: 'accessing'!

calendar
	"Vraci odkaz na kalendar udalosti simulovaneho modelu."

	^myCalendar!

endTime
	"Zpristupni konecnou hodnotu simulacniho casu."

	^endTime!

entities
	"Vraci celkovy pocet entit, ktere se vyskytly v simulacnim systemu 
	od zacatku simulace."

	^count!

entitiesNow
	"Vraci aktualni pocet entit v systemu."

	^actCount!

flag
	"Zpristupni priznak zastaveni simulace - je-li simulace ukoncena, pak 
	odkazuje na promennou 'true'."

	^stopFlag!

nextTime
	"Zpristupni nasledujici hodnotu simulacniho casu."

	^nextTime!

nextTime: nTime
	"Nastavi nasledujici hodnotu simulacniho casu."

	nextTime := nTime asDouble!

phase
	"Informuje o aktualni fazi simulace"

	^phase!

startTime
	"Zpristupni pocatecni hodnotu simulacniho casu."

	^startTime!

time
	"Vraci aktualni hodnotu simulacniho casu."

	^simTime! !

!MySimulation methodsFor: 'printing'!

output: aStream
	"Prezentace vysledku simulace na vystupu specifikovanem
	argumentem 'aStream'. Tuto metodu je vhodne predefinovat v
	subtridach podle pozadovanych vystupnivh informaci."

	aStream nextPutAll: 'Simulacni model `' , self class name , '`'; cr; nextPutAll: 'Simulacni cas: '.
	simTime printOn: aStream.
	aStream cr; nextPutAll: 'Planovany cas ukonceni simulace: '.
	endTime printOn: aStream.
	aStream cr; endEntry!

warning: aString on: aStream time: aTime 
	"Pomocny vypis na Transcript."

	aStream nextPutAll: 'WARNING:  '; nextPutAll: aString; nextPutAll: '  '.
	aTime printOn: aStream.
	aStream nextPutAll: '.'.
	aStream cr; endEntry! !

!MySimulation methodsFor: 'error handling'!

error: errorMessage
	"Osetri vyskyt chyby behem simulace - vypise chybovou hlasku do
	okna na obrazovku a ceka na pokyn uzivatele k zastaveni simulace
	nebo jejim pokracovani (pak je nutno chybu odstranit s pomoci
	ladicich prostredku Smalltalku)."

	(self confirm: ' CHYBA !!
 ' , self class name , ' - ' , errorMessage , '
 Skoncit simulaci ?')
		ifTrue: [self stop]
		ifFalse: [super error: errorMessage]! !

!MySimulation methodsFor: 'user interface'!

inspect
	"Zajisti otevreni inspektoru pro tento simulacni model (umoznuje
	sledovat vsechny jeho atributy)."

	SimInspectorView open: ((SimModelInspector inspect: self)
			initialize: self)
		label: 'Model ze tridy `' , self class name , '`'.!

variables
	"Vraci seznam vsech instancnich promennych tohoto modelu (pro
	sledovani jejich hodnot v inspektoru)."

	self class == (Smalltalk at: #MySimulation) ifFalse: [^self class instVarNames]
		ifTrue: [^nil]! !

!MySimulation methodsFor: 'cont simulation'!

continueInit
	"Inicializuje spojitou cast modelu."

	maxStep = nil ifTrue: [maxStep := (endTime - startTime / 1000) asDouble].
	minStep = nil ifTrue: [minStep := (endTime / 1000000000000000) asDouble].
	absError = nil ifTrue: [absError := 0 asDouble].
	relError = nil ifTrue: [relError := (1 / 100000) asDouble].
	optStep := maxStep.
	stepStartTime := simTime.
	method = nil ifTrue: [method := 'RKE4'].
	conditionFlag := false.
	self isAnyAll
		ifTrue: 
			[self integrInit.
			self statusInit.
			self condInit.
			self dynamic.
			dynamicFlag := true.
			self condTest.
			dynamicFlag := false.
			self condSet]!

contractStep
	"Provede zkraceni kroku spojite simulace na implicitni hodnotu."

	contractStepFlag := true!

contractStep: newTime 
	"Provede zkraceni kroku spojite simulace. 
	Promenna 'newTime' - cas, na ktery je treba dokrocit."

	| step |
	contractStepFlag := true.
	step := (newTime - stepStartTime) asDouble.
	step < contractStep ifTrue: [contractStep := step].
	step < minStep ifTrue: [contractStep := minStep]!

doCondition
	"Provede reakci na zmenu podminek."

	conditionFlag
		ifTrue: 
			[self condAction.
			self condSet.
			resetStatus := true]!

dynamic
	"Provede vyhodnoceni integratoru a stavovych bloku."

	self statusClearValueOK.
	self statusEvaluate.
	self integrEvaluate!

isAny
	"Kontroluje, zda byly zadane nejake integratory nebo stavove 
	nelinearity."

	integrator isEmpty not | status isEmpty not ifTrue: [^true].
	^false!

isAnyAll
	"Kontroluje, zda byly zadane nejake integratory, stavove 
	nelinearity nebo podminky."

	integrator isEmpty not | status isEmpty not | condition isEmpty not ifTrue: [^true].
	^false!

stepSimulation
	"Provede jeden krok spojite simulace."

	method = 'RKE4' ifTrue: [self rungeKuttaEngland4]! !

!MySimulation methodsFor: 'cont simulation acces'!

con
	"Vraci hodnotu promenne, ktera indikuje zmenu v podminkach."

	^conditionFlag!

con: newCondition 
	"Nastavuje hodnotu promenne, ktera indikuje zmenu v podminkach."

	conditionFlag := newCondition!

conStep
	"Vraci promennou pro zkraceni kroku spojite simulace."

	^contractStep!

conStep: newValue 
	"Nastavuje promennou pro zkraceni kroku spojite simulace."

	contractStep := newValue!

dyn
	"Vraci hodnotu promenne, ktera indikuje, ze jsme uvnitr kroku spojite 
	simulace."

	^dynamicFlag!

graph
	"Vraci pole odkazu na grafy."

	^graph!

reset: newValue 
	"Nastavuje promennou resetStatus - novy start simulace."

	resetStatus := newValue!

stepNum
	"Vraci hodnotu poctu kroku spojite simulace."

	^stepNum! !

!MySimulation methodsFor: 'cont simulation elem'!

condAction
	"Testuje vsechny podminky."

	| index |
	condition isEmpty
		ifFalse: 
			[index := condition size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(condition at: index) test ifTrue: [(condition at: index) action]]]!

condInit
	"Inicializuje vsechny podminky."

	| index |
	condition isEmpty
		ifFalse: 
			[index := condition size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(condition at: index) initVal]]!

condNew: aCondition 
	"Zaradi dalsi podminku do pole."

	condition := condition copyWith: aCondition.
	^aCondition!

condSet
	"Nastavuje vsechny podminky."

	| index |
	condition isEmpty
		ifFalse: 
			[index := condition size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(condition at: index) setValue]]!

condTest
	"Testuje vsechny podminky."

	| index |
	condition isEmpty
		ifFalse: 
			[index := condition size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(condition at: index) test ifTrue: [conditionFlag := true]]]!

graphAction
	"Zapisuje vystupy spojite simulace do grafu. A to bud jen do souboru 
	nebo jen na obrazovku nebo do obou soucasne."

	| index pom help |
	graph isEmpty
		ifFalse: 
			[index := graph size + 1.
			[(index := index - 1) > 0]
				whileTrue: 
					[pom := (help := graph at: index) test.
					nextTime = help time
						ifTrue: 
							[pom = 0 | (pom = 1) ifTrue: [help file].
							pom = 0 | (pom = 2) ifTrue: [help graph].
							simTime + (help step * 0.99) > endTime
								ifTrue: [help stop]
								ifFalse: [simTime + help step > endTime
										ifTrue: 
											[(SimProcess new: [MySimulation active graphAction])
												activateAt: endTime.
											help time: endTime]
										ifFalse: 
											[(SimProcess new: [MySimulation active graphAction])
												activateAt: simTime + help step.
											help time: simTime + help step]]]]]!

graphInit
	"Inicializuje vystupy spojite simulace."

	| index pom help |
	graph isEmpty
		ifFalse: 
			[index := graph size + 1.
			[(index := index - 1) > 0]
				whileTrue: 
					[pom := (help := graph at: index) test.
					stepNum = 0
						ifTrue: 
							[pom = 0 | (pom = 1)
								ifTrue: 
									[help firstHeader.
									help file].
							pom = 0 | (pom = 2)
								ifTrue: 
									[help view
										start: startTime
										end: endTime
										text: help text.
									help graph].
							(SimProcess new: [MySimulation active graphAction])
								activateAt: startTime + help step.
							help time: startTime + help step]]]!

graphNew: aGraph 
	"Zaradi dalsi graf do pole."

	graph := graph copyWith: aGraph.
	^aGraph!

integrEvaluate
	"Vyhodnoti vsechny integratory."

	| index |
	integrator isEmpty
		ifFalse: 
			[index := integrator size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(integrator at: index) eval]]!

integrInit
	"Inicializuje vsechny integratory."

	| index |
	integrator isEmpty
		ifFalse: 
			[index := integrator size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(integrator at: index) initVal]]!

integrIsAny
	"Kontroluje, zda byly zadane nejake integratory."

	integrator isEmpty not ifTrue: [^true].
	^false!

integrLtoN
	"Obnova stavu vsech integratoru."

	| index |
	integrator isEmpty
		ifFalse: 
			[index := integrator size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(integrator at: index) lTon]]!

integrNew: aIntegrator 
	"Zaradi dalsi podminku do pole."

	integrator := integrator copyWith: aIntegrator.
	^aIntegrator!

integrNtoL
	"Obnova stavu vsech integratoru."

	| index |
	integrator isEmpty
		ifFalse: 
			[index := integrator size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(integrator at: index) nTol]]!

statusClearValueOK
	"Inicializuje u vsech stavovych bloku promennou 'ValueOK'."

	| index |
	status isEmpty
		ifFalse: 
			[index := status size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(status at: index) clear]]!

statusEvaluate
	"Vyhodnoti vsechny stavove bloky."

	| index |
	status isEmpty
		ifFalse: 
			[index := status size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(status at: index) eval]]!

statusInit
	"Inicializuje vsechny stavove bloky."

	| index |
	status isEmpty
		ifFalse: 
			[index := status size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(status at: index)
						initVal: 0]]!

statusLtoN
	"Obnova stavu vsech stavovych bloku."

	| index |
	status isEmpty
		ifFalse: 
			[index := status size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(status at: index) lTon]]!

statusNew: aStatus 
	"Zaradi dalsi stavovy blok do pole."

	status := status copyWith: aStatus.
	^aStatus!

statusNtoL
	"Uschova stavu vsech stavovych bloku."

	| index |
	status isEmpty
		ifFalse: 
			[index := status size + 1.
			[(index := index - 1) > 0]
				whileTrue: [(status at: index) nTol]]! !

!MySimulation methodsFor: 'cont simulation set'!

setAccuracy: relErr 
	"Nastaveni povolene chyby (relativni, absolutni) numericke integrace."

	self setAccuracy: 0 relError: relErr!

setAccuracy: absErr relError: relErr 
	"Nastaveni povolene chyby (relativni, absolutni) numericke integrace."

	absError := absErr asDouble.
	relErr > 1
		ifTrue: [relError := 1 asDouble]
		ifFalse: [relError := relErr asDouble].
	relError < (1 / 10000000000000) ifTrue: [self error: 'Nelze zajistit zadanou presnost numericke integrace']!

setMethod: newMethod 
	"Nastaveni metody numericke integrace."

	newMethod = 'RKE4'
		ifTrue: [method := newMethod]
		ifFalse: [method := 'RKE4']!

setStep: minSt to: maxSt 
	"Nastaveni povoleneho rozsahu kroku spojite simulace."

	maxStep = nil
		ifTrue: [maxStep := (endTime - startTime / 1000) asDouble]
		ifFalse: [maxStep := maxSt asDouble].
	minStep = nil
		ifTrue: [minStep := (endTime / 1000000000000000) asDouble]
		ifFalse: [minStep := minSt asDouble].
	minStep > maxStep ifTrue: [self error: 'Chybne parametry u nastavovani kroku spojite simulace'].
	minStep < (endTime / 1000000000000000) ifTrue: [minStep := (endTime / 1000000000000000) asDouble]! !

!MySimulation methodsFor: 'num integration'!

rungeKuttaEngland4
	"Metoda numericke integrace Runge-Kutta-England 4. radu."

	| isEndStepEvent half qrt index a1 a2 a3 a4 a5 a6 a7 pom wasContractStepFlag doubleStepFlag si di xi errorS eerr terr break helpFlag help numIntg |
	index := integrator size.
	a1 := Array new: index.
	a2 := Array new: index.
	a3 := Array new: index.
	a4 := Array new: index.
	a5 := Array new: index.
	a6 := Array new: index.
	a7 := Array new: index.
	si := Array new: index.
	di := Array new: index.
	xi := Array new: index.
	sizeStep := optStep asDouble.
	isEndStepEvent := simTime + (1.01 * sizeStep) >= nextTime.
	isEndStepEvent ifTrue: [sizeStep := nextTime - simTime].
	stepStartTime := simTime asDouble.
	helpFlag := false.
	resetStatus
		ifTrue: 
			["inicializace kroku"
			self integrNtoL.
			self statusNtoL.
			self dynamic.
			self condTest.
			self integrNtoL.
			self statusNtoL.
			resetStatus := false.
			helpFlag := true].
	help := conditionFlag & helpFlag.
	help
		ifFalse: 
			[helpFlag := true.
			sizeStep <= 0 ifTrue: [self error: 'Interni chyba numericke integrace krok <= 0.'].
			self integrIsAny ifTrue: ["zacatek kroku"
				[helpFlag]
					whileTrue: 
						[[helpFlag]
							whileTrue:
								[[helpFlag]
									whileTrue: 
										[helpFlag := false.
										sizeStep := sizeStep max: minStep.
										half := 0.5 asDouble * sizeStep.
										qrt := 0.5 asDouble * half.
										contractStepFlag := false.
										contractStep := qrt.
										numIntg := integrator size + 1.
										index := numIntg.
										[(index := index - 1) > 0]
											whileTrue: 
												[pom := integrator at: index.
												a1 at: index put: half * pom ddl.
												pom ss: pom ssl + (0.5 asDouble * (a1 at: index))].	"1/4 kroku"
										simTime := stepStartTime + qrt.
										self dynamic.
										index := numIntg.
										[(index := index - 1) > 0]
											whileTrue: 
												[pom := integrator at: index.
												a2 at: index put: half * pom dd.
												pom ss: pom ssl + (0.25 asDouble * ((a1 at: index) + (a2 at: index)))].
										self dynamic.
										index := numIntg.
										[(index := index - 1) > 0]
											whileTrue: 
												[pom := integrator at: index.
												a3 at: index put: half * pom dd.
												pom ss: pom ssl - (a2 at: index) + (2 asDouble * (a3 at: index))].	"1/2 kroku"
										simTime := stepStartTime + half.
										self dynamic.
										index := numIntg.
										[(index := index - 1) > 0]
											whileTrue: 
												[pom := integrator at: index.
												a4 at: index put: half * pom dd.
												pom ss: pom ssl + ((a1 at: index)
															+ (4 asDouble * (a3 at: index)) + (a4 at: index) / 6 asDouble)].
										self condTest.
										contractStepFlag & (sizeStep > minStep)
											ifTrue: 
												[sizeStep := contractStep asDouble.
												isEndStepEvent := false.
												helpFlag := true.
												contractStepFlag := false]].	"bez na zacatek numericke integrace"
								wasContractStepFlag := contractStepFlag.
								contractStepFlag := false.
								contractStep := half.
								index := numIntg.
								["uschova hodnot v 1/2 kroku"
								(index := index - 1) > 0]
									whileTrue: 
										[pom := integrator at: index.
										di at: index put: pom dd.
										si at: index put: pom ss].
								index := status size + 1.
								[(index := index - 1) > 0]
									whileTrue: [xi at: index put: (status at: index) st].
								self dynamic.
								index := numIntg.
								[(index := index - 1) > 0]
									whileTrue: 
										[pom := integrator at: index.
										a5 at: index put: half * pom dd.
										pom ss: (si at: index)
												+ (0.5 asDouble * (a5 at: index))].	"3/4 kroku"
								simTime := stepStartTime + half + qrt.
								self dynamic.
								index := numIntg.
								[(index := index - 1) > 0]
									whileTrue: 
										[pom := integrator at: index.
										a6 at: index put: half * pom dd.
										pom ss: (si at: index)
												+ (0.25 asDouble * ((a5 at: index)
															+ (a6 at: index)))].
								self dynamic.
								index := numIntg.
								[(index := index - 1) > 0]
									whileTrue: 
										[pom := integrator at: index.
										a7 at: index put: half * pom dd.
										pom ss: pom ssl + ((a1 at: index) negated - (96 asDouble * (a2 at: index)) + (92 asDouble * (a3 at: index)) - (121 asDouble * (a4 at: index)) + (144 asDouble * (a5 at: index)) + (6 asDouble * (a6 at: index)) - (12 asDouble * (a7 at: index)) / 6 asDouble)].	"konec kroku KONTROLA PRESNOSTI 
								NUMERICKE 
								INTEGRACE"
								simTime := stepStartTime + sizeStep.
								self dynamic.
								doubleStepFlag := true.
								errorS := 0.
								break := false.
								index := numIntg.
								[(index := index - 1) > 0 & break not]
									whileTrue: 
										[eerr := ((a1 at: index) negated + (4 asDouble * (a3 at: index)) + (17 asDouble * (a4 at: index)) - (23 asDouble * (a5 at: index)) + (4 asDouble * (a7 at: index)) - (half * (integrator at: index) dd asDouble) / 90 asDouble) abs.
										terr := absError asDouble + (relError asDouble * (si at: index)) abs.
										eerr < (0.02 * terr) ifFalse: [eerr > terr
												ifTrue: 
													["povolena tolerance 
													prekrocena"
													sizeStep > minStep
														ifTrue: 
															["lze zkratit 
															krok"
															optStep := 0.5 asDouble * sizeStep max: minStep.
															sizeStep := optStep asDouble.
															isEndStepEvent := false.
															helpFlag := true].
													helpFlag
														ifFalse: 
															[errorS := errorS + 1.
															conditionFlag ifTrue: ["nelze 
																zkratit krok, udalost 1/2 
																kroku, nelze kratit a presnost 
																mimo toleranci"
																break := true]].
													doubleStepFlag := false]]]].
						errorS > 0 ifTrue: ["bez na zacatek numericke integrace"
							self
								warning: 'Nelze zajistit zadanou presnost numericke integrace'
								on: Transcript
								time: simTime].
						wasContractStepFlag
							ifTrue: 
								["konec kontroly presnosti"
								"obnova stavu v 1/2 kroku"
								index := numIntg.
								[(index := index - 1) > 0]
									whileTrue: 
										[pom := integrator at: index.
										pom dd: (di at: index).
										pom ss: (si at: index)].
								index := status size + 1.
								[(index := index - 1) > 0]
									whileTrue: [(status at: index)
											st: (xi at: index)].
								simTime := stepStartTime + half.
								isEndStepEvent := false]
							ifFalse: 
								["posune zacatek kroku do 1/2 a 
								pokracuje"
								index := numIntg.
								[(index := index - 1) > 0]
									whileTrue: 
										[pom := integrator at: index.
										pom ddl: (di at: index).
										pom ssl: (si at: index).
										pom ss: (si at: index)
												- (a6 at: index) + (2 asDouble * (a7 at: index))].
								index := status size + 1.
								[(index := index - 1) > 0]
									whileTrue: [(status at: index)
											stl: (xi at: index)].
								stepStartTime := stepStartTime + half.
								self dynamic.
								index := numIntg.
								[(index := index - 1) > 0]
									whileTrue: 
										[pom := integrator at: index.
										pom ss: (si at: index)
												+ ((a5 at: index) + (4 asDouble * (a7 at: index)) + (half * pom dd) / 6)].
								self condTest.
								contractStepFlag & (sizeStep > minStep)
									ifTrue: 
										["zkraceni kroku"
										sizeStep := contractStep asDouble.
										contractStepFlag := false.
										isEndStepEvent := false.
										helpFlag := true]]]]].
	help
		ifFalse: 
			[self integrIsAny
				ifFalse: 
					["Model bez integratoru"
					break := true.
					[break]
						whileTrue: 
							[break := false.
							sizeStep := minStep max: sizeStep.
							contractStepFlag := false.
							contractStep := 0.5 asDouble * sizeStep.
							simTime := stepStartTime + sizeStep.
							self dynamic.
							self condTest.
							contractStepFlag not ifTrue: [break := true].
							break
								ifFalse: 
									[sizeStep <= minStep ifTrue: [break := true].
									break
										ifFalse: 
											[isEndStepEvent := false.
											sizeStep := contractStep asDouble.
											self statusLtoN]]]].	"novy stav"
			stepStartTime := simTime asDouble.
			self integrNtoL.
			self statusNtoL.
			isEndStepEvent ifTrue: [simTime := nextTime].
			self integrIsAny
				ifTrue: 
					[doubleStepFlag ifTrue: [optStep := optStep asDouble + optStep asDouble].
					optStep := optStep asDouble min: maxStep]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MySimulation class
	instanceVariableNames: ''!


!MySimulation class methodsFor: 'instance creation'!

new
	"Vytvari novy simulacni model a inicializuje jeho charakteristiky"

	^super new initialize! !

!MySimulation class methodsFor: 'initialization'!

initialize
	"Inicializace promennych tridy simulacnich modelu."

	SuspendedQueue := OrderedCollection new.
	ActiveSimulation := nil! !

!MySimulation class methodsFor: 'process handling'!

addProcess: newProcess with: newModel 
	"Vlozi do fronty polozku s odkazem na novy proces 'newProcess' 
	a jeho simulacni model 'newModel'. Proces realizujici simulacni beh 
	je spusten (volana po vzniku a aktivaci noveho modelu, promenna 
	'ActiveSimulation' tedy odkazuje na tento model."

	| item |
	SuspendedQueue isEmpty
		ifFalse: 
			[item := SuspendedQueue first.
			ActiveSimulation == (item at: 2)
				ifTrue: 
					[(item at: 1) suspend.
					ActiveSimulation := newModel]].
	item := Array with: newProcess with: newModel.
	SuspendedQueue addFirst: item.
	newProcess resume!

cancelActive
	"Ukonci prave aktivni simulaci a zrusi prislusnou polozku 
	z fronty pozastavenych simulaci. Soucasne je rizeni 
	predano dalsimu modelu z fronty; je-li fronta prazdna, 
	zadny model neni aktivovan."

	| item |
	SuspendedQueue isEmpty
		ifTrue: 
			[ActiveSimulation := nil.
			^true].
	(SuspendedQueue first at: 2)
		== ActiveSimulation ifTrue: [SuspendedQueue removeFirst].
	SuspendedQueue isEmpty
		ifTrue: 
			[ActiveSimulation := nil.
			^true]
		ifFalse: 
			[item := SuspendedQueue first.
			ActiveSimulation := item at: 2.
			(item at: 1) resume]!

clear
	"Ukonci postupne vsechny pozastavene simulace a uvede tak 
	promenne tridy a tim i cely simulacni system do pocatecniho stavu."

	[SuspendedQueue isEmpty]
		whileFalse: 
			[| item |
			item := SuspendedQueue first.
			(item at: 2) stop].
	ActiveSimulation ~~ nil
		ifTrue: [ActiveSimulation stop]
		ifFalse: [ActiveSimulation := nil]!

suspendActive
	"Deaktivuje prave probihajici simulaci, vybere dalsi model z fronty 
	pozastavenych simulaci a preda mu rizeni (aktivuje jej). Je-li 
	fronta prazdna, preda rizeni zpet odesilateli (aktivnimu modelu). 
	Modelu, ktery nebyl spusten na pozadi, se vraci rizeni hned zpatky - 
	ostatni modely cekaji na ukonceni jeho behu."

	| item item1 |
	SuspendedQueue isEmpty ifTrue: [^true].
	(SuspendedQueue first at: 2)
		== ActiveSimulation        "jen je-li volana ze simulacniho modelu spusteneho na pozadi"
		ifTrue: 
			[item := SuspendedQueue removeFirst.
			 SuspendedQueue isEmpty
				ifTrue:              "fronta je prazdna - rizeni je vraceno zpet"
					[SuspendedQueue add: item.
					^true]
				ifFalse:             "aktivace modelu z cela fronty"
					[SuspendedQueue addLast: item.
					item1 := item at: 1.
					item := SuspendedQueue first.
					ActiveSimulation := item at: 2.
					(item at: 1) resume.
					^item1 suspend]]! !

!MySimulation class methodsFor: 'accessing'!

active
	"Zpristupni aktivni (prave bezici) simulacni model."

	^ActiveSimulation!

time
	"Vraci aktualni hodnotu simulacniho casu v prave aktivnim simulacnim 
	modelu."

	ActiveSimulation isNil ifFalse: [^ActiveSimulation time]
		ifTrue: [^nil]! !

MySimulation initialize!


Object subclass: #Facility
	instanceVariableNames: 'qFlag queueIn entityIn fName tstat '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
Facility comment:
'Trida Facility definuje chovani a atributy pasivniho objektu typu obsluzne zarizeni, ktery umoznuje modelovat prvky systemu s vylucnym pristupem. Zarizeni muze tedy obsadit v danem okamziku jen jeden aktivni objekt (entita), ostatni pozadavky museji cekat ve vstupni fronte k zarizeni. Razeni pozadavku v teto fronte je dano:
- jejich prioritou obsluhy
- jejich prioritou
- (implicitne) strategii FIFO
Implicitni vstupni fronta muze byt nahrazena specifikovanou externi frontou.
Uvolneni obsazeneho zarizeni musi provest entita, ktera jej obsadila, jinak dochazi k chybe.
Kazde zarizeni obsahuje prostredky pro sber statistik, ktere sleduji jeho dulezite charakteristiky (vyuziti, pocet pozadavku, ...).

Promenne instance:
  queueIn <Queue>	- odkaz na vstupni frontu
  qFlag <Symbol>	- typ vstupni fronty (interni, externi)
  entityIn <SimProcess> - odkaz na obsluhovanou entitu 
  fName <String>	- jmeno zarizeni
  tstat <TStat>	- casova statistika zarizeni'!


!Facility methodsFor: 'task language'!

busy
	"Informuje o obsazenosti zarizeni - u obsazeneho zarizeni vraci 
	hodnotu 'true'"

	^entityIn ~= nil!

release: entity 
	"Uvolneni zarizeni obsluhovanou entitou. Zada-li o uvolneni zarizeni 
	jina entita, nez obsluhovana, dochazi k chybe. Uvolneni zarizeni je 
	zaznamenano do jeho statistiky. Po ukonceni obsluhy entity je 
	zahajena obsluha entity z cela vstupni fronty zarizeni."

	self busy ifFalse: [MySimulation active error: 'Pokus o uvolneni zarizeni, ktere neni obsazeno - ' , fName].
	entityIn == entity ifFalse: [MySimulation active error: 'Pokus o uvolneni zarizeni entitou, ktera jej neobsadila'].
	entityIn sPriority: 0.
	entityIn := nil.
	tstat rec: 0; decnum.
	queueIn isEmpty
		ifFalse: 
			["vyber dalsi entity z fronty"
			entityIn := queueIn removeFirst.
			tstat rec: 1.
			entityIn activate]!

seize: entity 
	"Zadost o obsluhu entity s prioritou 0. Je-li zarizeni volne, entita jej 
	obsadi a jeho obsazeni je zaznamenano do statistiky. V opacnem 
	pripade je entita pasivovana a umistena do vstupni fronty zarizeni."

	entity sPriority: 0.
	self busy
		ifTrue: 
			[queueIn add: entity.
			entity passivate]
		ifFalse: 
			[entityIn := entity.
			tstat rec: 1.0]!

seize: entity priority: spriority 
	"Zadost o obsluhu entity s prioritou 'spriority'. Je-li zarizeni volne, 
	entita jej obsadi a jeho obsazeni je zaznamenano do statistiky. V 
	opacnem pripade je entita pasivovana a umistena do vstupni fronty 
	zarizeni."

	entity sPriority: spriority.
	self busy
		ifTrue: 
			[queueIn add: entity.
			entity passivate]
		ifFalse: 
			[entityIn := entity.
			tstat rec: 1.0]! !

!Facility methodsFor: 'accessing'!

getName
	"Vraci jmeno zarizeni."

	^fName!

in
	"Zpristupni prave obsluhovanou entitu."

	^entityIn!

queueIn
	"Zpristupni vstupni frontu zarizeni."

	^queueIn!

setName: newName 
	"Nastavi nove jmeno zarizeni."

	fName := newName!

setQueue: newQueue 
	"Nastavi novou (externi) vstupni frontu zarizeni. Obsahuje-li puvodni 
	fronta nejake polozky, dochazi k chybe."

	qFlag = #internal
		ifTrue: 
			[queueIn size > 0 ifTrue: [MySimulation active error: 'Pokus o nahradu neprazdne vstupni fronty u zarizeni ' , fName].
			qFlag := #external].
	queueIn := newQueue!

tQueue
	"Vraci typ vstupni fronty."

	^qFlag!

tstat
	"Vraci odkaz na casovou statistiku zarizeni."

	^tstat! !

!Facility methodsFor: 'initialization'!

clear
	"Uvede do pocatecniho stavu vsechny atributy zarizeni, tj. zrusi 
	pripadnou obsluhu entity, inicializuje vstupni frontu a statistiku 
	zarizeni."

	queueIn clear.
	entityIn := nil.
	fName := ''.
	tstat clear!

initialize: name 
	"Inicializuje atributy zarizeni - jeho jmeno, vstupni frontu a jeji typ a 
	statistiku zarizeni."

	qFlag := #internal.	"interni fronta"
	queueIn := Queue newQ: 'Interni fronta zarizeni ' , name.
	entityIn := nil.
	fName := name.
	tstat := TStat new: 'Vyuziti zarizeni ' , fName! !

!Facility methodsFor: 'user interface'!

inspect: simModel 
	"Otevre okno inspektoru zarizeni, ktery umoznuje prohlizeni jeho 
	atributu, a nastavi v nem odkaz na simulacni model, v nemz se 
	zarizeni vyskytuje"

	SimInspectorView open: ((FacilityInspector inspect: self)
			initialize: simModel)
		label: 'Zarizeni `' , fName , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Facility class
	instanceVariableNames: ''!


!Facility class methodsFor: 'instance creation'!

new
	"Vytvari nove obsluzne zarizeni a aktivuje jeho inicializacni metodu."

	^super new initialize: ''!

new: fName 
	"Vytvari nove zarizeni s nazvem 'fName' a zajisti inicializaci jeho 
	atributu."

	^super new initialize: fName! !

Object subclass: #Store
	instanceVariableNames: 'capacity used sName queueIn qFlag tstat '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
Store comment:
'Trida Store definuje chovani a atributy pasivniho objektu typu sklad, ktery umoznuje modelovat prvky systemu se sdilenym pristupem. Sklad muze tedy obsadit vice entit soucasne, az do vycerpani jeho stanovene kapacity. Kazda entita pozaduje urcitou cast z celkova kapacity skladu a nema-li sklad tuto pozadovanou kapacitu volnou, je entita umistenado vstupni fronty skladu. Razeni pozadavku v teto fronte je dano:
 - jejich prioritou
- (implicitne) strategii FIFO
Implicitni vstupni fronta muze byt nahrazena specifikovanou externi frontou.
Uvolneni obsazene kapacity skladu realizuje entita, ktera tuto kapacitu obsadila. Uvolnena kapacita je k dispozici entitam cekajicim ve vstupni fronte (dosahne-li alespon velikosti pozadovane entitou z cela fronty, pak ji tato entita obsadi). Kazdy sklad obsahuje prostredky pro sber statistik, ktere sleduji jeho dulezite charakteristiky (prumerne vyuzita kapacita, pocet obslouzenych entit, ...).

Promenne instance:
  queueIn <Queue>	- odkaz na vstupni frontu
  qFlag <Symbol>	- typ vstupni fronty (interni, externi)
  capacity <Integer> - celkova kapacita skladu
  used <Integer>	- obsazena kapacita skladu
  sName <String>	- jmeno skladu
  tstat <TStat>	- casova statistika skladu'!


!Store methodsFor: 'initialization'!

clear
	"Uvede do pocatecniho stavu vsechny atributy skladu, tj. uvolni jeho 
	obsazenou kapacitu, inicializuje jeho vstupni frontu a statistiku"

	queueIn clear.
	used := 0.
	sName := ''.
	tstat clear!

initialize: name capacity: cap 
	"Inicializuje atributy skladu."

	qFlag := #internal.	"interni fronta"
	queueIn := Queue newQ: 'Interni fronta skladu ' , name.
	used := 0.
	capacity := cap.
	sName := name.
	tstat := TStat new: 'Casova statistika skladu ' , sName! !

!Store methodsFor: 'task language'!

empty
	"Vraci 'true', je-li sklad prazdny"

	^used = 0!

enter: entity capacity: rCap 
	"Zadost entity o obsazeni pozadovane kapacity 'rCap' skladu. Je-li 
	pozadovana kapacita vetsi, nez celkova, dochazi k chybe. Je-li 
	pozadovana kapacita volna, entita ji obsadi a provede se zaznam do 
	statistiky. V opacnem pripade je entita pasivovana a umistena do 
	vstupni fronty skladu."

	rCap > capacity ifTrue: [MySimulation active error: 'Pozadovana kapacita skladu ' , sName , ' je vetsi, nez celkova'].
	entity reqCap: rCap.
	self free < rCap
		ifTrue: 
			[queueIn add: entity.
			entity passivate]
		ifFalse:
			[used := used + rCap.
			tstat rec: used]!

full
	"Vraci 'true', je-li sklad plne obsazen."

	^used = capacity!

leave: rCap 
	"Uvolneni obsazene kapacity (rCap) skladu entitou. Je-li uvolnovana 
	kapacita vetsi, nez obsazovana, dochazi k chybe. Uvolneni kapacity 
	je zaznamenano do statistiky a volna kapacita je obsazovana 
	entitami z cela vstupni fronty, dokud je jeji velikost vetsi, nez je 
	kapacita pozadovana entitou z cela fronty."

	| item notOK |
	used < rCap | (rCap < 1) ifTrue: [MySimulation active error: 'Chyba pri uvolnovani kapacity skladu ' , sName].
	used := used - rCap.
	tstat rec: used; decnum.
	notOK := false.
	[queueIn isEmpty | self full | notOK]
		whileFalse: [queueIn first reqCap <= self free
				ifTrue: 
					[item := queueIn removeFirst.
					used := used + item reqCap.
					tstat rec: used.
					item activate]
				ifFalse: [notOK := true]]! !

!Store methodsFor: 'accessing'!

capacity
	"Vraci velikost celkove kapacity skladu"

	^capacity!

free
	"Vraci velikost volne kapacity skladu"

	^capacity - used!

getName
	"Vraci jmeno skladu."

	^sName!

queueIn
	"Zpristupni vstupni frontu skladu."

	^queueIn!

setCapacity: newCapacity 
	"Nastavi novou kapacitu skladu. Novou kapacitu lze nastavit 
	vpripade, ze je vetsi, nez puvodni nebo alespon vetsi, nez obsazena s 
	podminkou, ze vstupni fronta skladu je prazdna. V jinych pripadech 
	nastava chyba."

	capacity < newCapacity | (queueIn size = 0 & (used <= newCapacity))
		ifTrue: [capacity := newCapacity]
		ifFalse: [MySimulation active error: 'Nevhodne nastaveni nove kapacity skladu ' , sName]!

setName: newName 
	"Nastavi nove jmen skladu."

	sName := newName!

setQueue: newQueue 
	"Nastavi novou (externi) vstupni frontu skladu. Obsahuje-li puvodni 
	fronta nejake polozky (entity cekajici na obsluhu), dochazi k chybe."

	qFlag = #internal
		ifTrue: 
			[queueIn size > 0 ifTrue: [MySimulation active error: 'Pokus o nahradu neprazdne vstupni fronty u skladu ' , sName].
			qFlag := #external].
	queueIn := newQueue!

tstat
	"Zpristupni casovou statistiku skladu."

	^tstat!

used
	"Vraci velikost obsazene (pouzite) kapacity skladu."

	^used! !

!Store methodsFor: 'user interface'!

inspect: simModel 
	"Otevre okno inspektoru skladu, ktery umoznuje prohlizeni jeho 
	atributu, a nastavi v nem odkaz na simulacni model, v nemz se 
	sklad vyskytuje"

	SimInspectorView open: ((StoreInspector inspect: self)
			initialize: simModel)
		label: 'Sklad `' , sName , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Store class
	instanceVariableNames: ''!


!Store class methodsFor: 'instance creation'!

new: cap
	"Vytvari novy objekt typu sklad s kapacitou 'cap' a aktivuje metodu
	pro inicializaci jeho atributu."

	^super new initialize: '' capacity: cap!

new: sName capacity: cap
	"Vytvari novy objekt typu sklad s nazvem 'sName' a kapacitou 'cap'
	a aktivuje metodu pro inicializaci jeho atributu."

	^super new initialize: sName capacity: cap! !

Object subclass: #TStat
	instanceVariableNames: 'sumXt min max t0 t1 x1 num sName '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
TStat comment:
'Trida TStat definuje charakteristiky a akce objektu, ktere jsou v simulacnim systemu urceny ke sledovani casoveho prubehu hodnot sledovanych velicin a jejich zpracovani. Jeji instance poskytuji udaje typu:
- maximalni, minimalni a prumerna hodnota vstupu
- pocet vstupnich hodnot
- pocatecni cas
- posledni zaznamenana hodnota a cas jejiho zaznamu
- suma soucinu hodnoty a prislusneho casoveho intervalu
Ze zaznamenanych udaju lze urcit prumernou hodnotu vstupu za casovy interval.
 
Promenne  instance: 
  num <Integer> - pocet vstupnich hodnot
  t0 <Float> 	- pocatecni cas
  max <Float> 	- maximalni vstupni hodnota
  min <Float> 	- minimalni vstupni hodnota
  x1 <Float>	- posledni zaznamenana hodnota
  t1 <Float> 	- cas zaznamu teto posledni hodnoty
  sName <String> - oznaceni statistiky'!


!TStat methodsFor: 'recording'!

rec: x 
	"Zaznamena vstupni hodnotu x a aktualni cas do casove statistiky, 
	tj. aktualizuje hodnoty jejich atributu."

	| pom |
	MySimulation time < t1 ifTrue: [self clear].
	pom := MySimulation time - t1 * x1.
	sumXt := sumXt + pom.
	x1 := x.
	t1 := MySimulation time.
	num := num + 1.
	num = 1
		ifTrue: 
			[min := x.
			max := x]
		ifFalse: 
			[x < min ifTrue: [min := x].
			x > max ifTrue: [max := x]]! !

!TStat methodsFor: 'accessing'!

decnum
	"Snizi pocet zaznamenanych hodnot do casove statistiky o 1 (pouzito 
	ke korekci pri zaznamu)."

	num := num - 1!

getName
	"Vraci jmeno casove statistiky."

	^sName!

max
	"Vraci maximum ze zaznamenanych vstupnich hodnot."

	^max!

min
	"Vraci minimum ze zaznamenanych vstupnich hodnot."

	^min!

num
	"Vraci pocet zaznamenanych hodnot."

	^num!

setName: newName 
	"Nastavi nove jmeno casove statistiky."

	sName := newName!

sumXt
	"Vraci sumu soucinu hodnoty a prislusneho casoveho intervalu."

	^sumXt!

tInit
	"Vraci pocatecni cas."

	^t0!

tLast
	"Vraci cas posledniho zaznamu do statistiky."

	^t1!

xLast
	"Vraci posledni zaznamenanou hodnotu."

	^x1! !

!TStat methodsFor: 'initialization'!

clear
	"Uvede vsechny atributy casove statistiky do pocatecniho stavu."

	self initialize: sName!

initialize: name 
	"Inicializuje vsechny atributy casove statistiky."

	sName := name.
	sumXt := 0.
	min := 0.
	max := 0.
	num := 0.
	t0 := MySimulation time.
	t1 := t0.
	x1 := 0! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TStat class
	instanceVariableNames: ''!


!TStat class methodsFor: 'instance creation'!

new
	"Vytvari novou casovou statistiku a inicializuje jeji atributy."

	^super new initialize: ''!

new: sName 
	"Vytvari novou casovou statistiku se jmenem 'sName' a inicializuje jeji atributy."

	^super new initialize: sName! !

Object variableSubclass: #Queue
	instanceVariableNames: 'queue qName statN statDT '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulation'!
Queue comment:
'Trida Queue definuje frontu pro razeni polozek (entit) u pasivnich objektu simulovanych systemu. Kriteria razeni entit:
1. priorita obsluhy entity
2. priorita entity
3. (implicitne) strategie FIFO
Fronta pouziva ke sve reprezentaci instanci tridy ''SortedCollection'' ulozenou ve sve inst. promenne ''queue''. Vsechny operace s polozkami, vcetne definice razeni, jsou tedy provadeny s pomoci operaci definovanych ve tride ''SortedCollection''. Soucasti fronty jsou dve statistiky - statistika poctu entit ve fronte a statistika jejich dob cekani, s jejichz pomoci lze urcit dulezite charakteristiky fronty - napr.: prumernou delku fronty, prumernou dobu cekani, ... .

Promenne instance:
  queue <SortedCollection> - reprezentuje celou frontu
  qName <String> 			- jmeno fronty
  statN	<Stat>				- statistika poctu entit ve fronte
  statDT <TStat>			- casova statistika dob cekani entit ve fronte'!


!Queue methodsFor: 'accessing'!

first
	"Zpristupni prvni polozku fronty."

	^queue first!

getName
	"Vraci jmeno fronty."

	^qName!

isEmpty
	"Poskytuje informaci o stavu fronty - vraci 'true', je-li fronta 
	prazdna."

	^queue isEmpty!

last
	"Zpristupni posledni polozku fronty."

	^queue last!

setName: newName 
	"Nastavi nove jmeno fronty."

	qName := newName!

size
	"Vraci pocet prvku fronty."

	^queue size!

statDT
	"Zpristupni statistiku delky cekani ve fronte."

	^statDT!

statN
	"Zpristupni statistiku pro delku fronty."

	^statN! !

!Queue methodsFor: 'initialization'!

clear
	"Uvede do pocatecniho stavu vsechny atributy fronty (frontu a obe 
	statistiky)."

	[self isEmpty]
		whileFalse: [queue removeLast].
	qName := ''.
	statN clear.
	statDT clear!

initialize: qname 
	"Inicializuje atributy instance - vytvori frontu s definovanym 
	usporadanim, inicializuje obe jeji statistiky a pojmenuje ji."

	queue := SortedCollection sortBlock: [:x :y | x sPriority >= y sPriority & (x priority >= y priority)].
	qName := qname.
	statN := TStat new: 'Pocet prvku fronty ' , qName.
	statDT := Stat new: 'Doba cekani prvku ve fronte ' , qName! !

!Queue methodsFor: 'removing'!

remove: entity 
	"Zrusi polozku 'entity' z fronty a vlozi udaje o delce fronty a dobe 
	cekani teto entity ve fronte do obou statistik."

	| item |
	item := queue remove: entity ifAbsent: [^nil].
	statN rec: self size; decnum.
	statDT rec: MySimulation time - entity inpTime.
	^item!

removeFirst
	"Zrusi prvni polozku z fronty a vlozi udaje o delce fronty a dobe 
	cekani teto entity ve fronte do obou statistik."

	statN rec: self size; decnum.
	statDT rec: MySimulation time - self first inpTime.
	^queue removeFirst!

removeLast
	"Zrusi posledni polozku z fronty a vlozi udaje o delce fronty a dobe 
	cekani teto entity ve fronte do obou statistik."

	statN rec: self size; decnum.
	statDT rec: MySimulation time - self last inpTime.
	^queue removeLast! !

!Queue methodsFor: 'adding'!

add: entity 
	"Vlozi polozku entity do fronty podle danych kriterii (priorita obsluhy, 
	priorita a strategie FIFO). Nastavi cas vstupu entity do fronty a 
	vlozi udaj o delce fronty do jeji statistiky."

	statN rec: self size.
	entity inpTime: MySimulation time.
	queue add: entity! !

!Queue methodsFor: 'user interface'!

inspect: simModel 
	"Otevre okno inspektoru fronty, ktery umoznuje prohlizeni jejich 
	atributu, a nastavi v nem odkaz na simulacni model, v nemz se 
	fronta vyskytuje"

	SimInspectorView open: ((QueueInspector inspect: self)
			initialize: simModel)
		label: 'Fronta `' , qName , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Queue class
	instanceVariableNames: ''!


!Queue class methodsFor: 'instance creation'!

newQ
	"Vytvari novy objekt a inicializuje jeho atributy (nova fronta je 
	polozkou tohoto objektu)"

	^super new initialize: ''!

newQ: qname 
	"Vytvari novy objekt se jmenem 'qname' a inicializuje jeho atributy 
	(nova fronta je polozkou tohoto objektu)"

	^super new initialize: qname! !