Object subclass: #BasicRandom
	instanceVariableNames: ''
	classVariableNames: 'Basic '
	poolDictionaries: ''
	category: 'Probability Distributions'!
BasicRandom comment:
'Trida ''BasicRandom'' predstavuje generator rovnomerneho rozlozeni z intervalu <0,1). Tvori zaklad hierarchie trid definujicich generatory nahodnych cisel v simulacnim systemu.

Promenna tridy:
  Basic <Random> - instance tridy ''Random'' predstavujici generator cisel v intervalu <0,1); dalsi cislo je generovano pri prijeti zpravy ''next'' (Basic next).'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BasicRandom class
	instanceVariableNames: ''!


!BasicRandom class methodsFor: 'private'!

generate: rNumber 
	"Bazova metoda pro transformaci cisel ze zakladniho na pozadovany 
	typ rozlozeni. Predefinovana v kazde subtride, v zakladni verzi 
	neprovadi zadnou cinnost. Argument 'rNumber' predstavuje nahodne 
	cislo ze zakladniho intervalu <0,1)."

	^rNumber! !

!BasicRandom class methodsFor: 'class initialization'!

initialize
	"Inicializuje promennou tridy, ktera predstavuje generator 
	rovnomerneho rozlozeni pro interval <0,1)."

	Basic := Random new! !

!BasicRandom class methodsFor: 'instance creation'!

new
	"Vytvori novou instanci - nahodne cislo z prislusneho rozlozeni 
	(nejcasteji rovnomerne z intervalu <0,1))."

	^self generate: Basic next!

next
	"Vytvori novou instanci - nahodne cislo z prislusneho rozlozeni. K 
	tomu pouziva privatni transformacni metodu definovanou ve tride 
	generatoru nahodnych cisel pozadovaneho rozlozeni. Metoda je 
	volana ze subtrid teto tridy."

	^self generate: Basic next! !

BasicRandom initialize!


BasicRandom subclass: #Poisson
	instanceVariableNames: ''
	classVariableNames: 'Mu '
	poolDictionaries: ''
	category: 'Probability Distributions'!
Poisson comment:
'Trida ''Poisson'' predstavuje generator Poissonova rozlozeni s parametrem ''Mu'', ktery predstavuje jeho stredni hodnotu.

Promenne tridy:
  Mu <Float> - parametr Poissonova rozlozeni predstavujici jeho stredni hodnotu'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Poisson class
	instanceVariableNames: ''!


!Poisson class methodsFor: 'private'!

generate: x 
	"Metoda pro generovani cisla z Poissonova rozlozeni s vyuzitim 
	generatoru cisel rovnomerneho rozlozeni intervalu <0,1)."

	| p n q |
	p := Mu negated exp.
	n := 0.
	q := 1.0.
	
	[q := q * BasicRandom next.
	q >= p]
		whileTrue: [n := n + 1].
	^n!

setMean: p 
	"Nastaveni parametru Poissonova rozlozeni."

	Mu _ p! !

!Poisson class methodsFor: 'instance creation'!

mean: p 
	"Pozadavek na generovani noveho cisla z Poissonova rozlozeni s 
	parametrem p (predstavuje stredni hodnotu). Nejdrive je 
	nastavena promenna tridy predstavujici parametr Poissonova 
	rozlozeni a potom se provadi generovani cisla s vyuzitim generatoru 
	zakladniho rozlozeni (viz metoda generate: x)."

	p > 0.0
		ifTrue: 
			[self setMean: p.
			^self next]
		ifFalse: [MySimulation active error: 'Parametr u Poissonova rozlozeni musi byt vetsi nez 0']! !

BasicRandom subclass: #Uniform
	instanceVariableNames: ''
	classVariableNames: 'High Low '
	poolDictionaries: ''
	category: 'Probability Distributions'!
Uniform comment:
'Trida ''Uniform'' predstavuje generator rovnomerneho rozlozeni z intervalu <low, high). Nahodne cislo z intervalu <0,1) ziskane pomoci generatoru definovaneho v supertride je transformovano metodou inverzni transformace.

Promenne tridy:
  Low <Float> - dolni mez intervalu 
  High <Float> - horni mez intervalu'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Uniform class
	instanceVariableNames: ''!


!Uniform class methodsFor: 'private'!

generate: rNum 
	"Metoda pro transformaci nahodneho cisla rovnomerneho rozlozeni z 
	intervalu <0,1) (argument x) na cislo v intervalu <Low, High)."

	^Low + (rNum * (High - Low))!

setLow: begin high: end 
	"Nastavi parametry rovnomerneho rozlozeni predstavujici meze 
	intervalu gener. cisel."

	Low := begin.
	High := end! !

!Uniform class methodsFor: 'instance creation'!

from: begin to: end 
	"Pozadavek na generovani noveho cisla z rovnomerneho rozlozeni 
	intervalu <begin, end). Nejdrive jsou nastaveny promenne tridy 
	predstavujici meze intervalu a potom se provadi transformace cisla 
	ze zakladniho intervalu do pozadovaneho (viz metoda generate: x)."

	begin > end
		ifTrue: [MySimulation active error: 'Chybne parametry u generatoru rovnomerneho rozlozeni.']
		ifFalse: 
			[self setLow: begin high: end.
			^self next]! !

BasicRandom subclass: #Binomial
	instanceVariableNames: ''
	classVariableNames: 'N Prob '
	poolDictionaries: ''
	category: 'Probability Distributions'!
Binomial comment:
'Trida ''Binomial'' predstavuje generator binomickeho rozlozeni s parametry ''N'' (pocet nezavislych pokusu) a ''Prob'' (pravdepodobnost nastoupeni jevu v pokusu).

Promenne tridy:
  N <Integer> - parametr binomickeho rozlozeni predstavujici pocet nezavislych pokusu
  Prob <Float> - parametr binomickeho rozlozeni predstavujici pravdepodobnost nastoupeni jevu v pokusu'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Binomial class
	instanceVariableNames: ''!


!Binomial class methodsFor: 'instance creation'!

events: n prob: p 
	"Pozadavek na generovani noveho cisla z binomickeho rozlozeni s 
	parametry 'n' (pocet nezavislych pokusu) a 'p' (pravdepodobnost 
	nastoupeni jevu v jednom pokusu)."

	(p between: 0.0 and: 1.0)
		ifFalse: [MySimulation active error: 'Chybne zadana pravdepodobnost u binomickeho rozlozeni.']
		ifTrue: 
			[N := n truncated.
			Prob := p.
			^self next]! !

!Binomial class methodsFor: 'private'!

generate: x 
	"Metoda pro generovani nahodneho cisla z binomickeho rozlozeni 
	pomoci generatoru nahodnych cisel zakladniho rozlozeni (rovnomerne 
	z intervalu <0,1))."

	| t rand |
	t := 0.
	rand := x.
	N
		timesRepeat: 
			[t := t + (rand <= Prob
							ifTrue: [1]
							ifFalse: [0]).
			rand := BasicRandom next].
	^t! !

BasicRandom subclass: #Exponential
	instanceVariableNames: ''
	classVariableNames: 'Mu '
	poolDictionaries: ''
	category: 'Probability Distributions'!
Exponential comment:
'Trida ''Exponential'' predstavuje generator exponencialniho rozlozeni s parametrem ''Mu'', ktery predstavuje jeho stredni hodnotu. Nahodne cislo z intervalu <0,1) ziskane pomoci generatoru definovaneho v supertride je transformovano metodou inverzni transformace.

Promenne tridy:
  Mu <Float> - parametr exponencialniho rozlozeni predstavujici stredni hodnotu'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Exponential class
	instanceVariableNames: ''!


!Exponential class methodsFor: 'private'!

generate: x 
	"Metoda pro transformaci nahodneho cisla ze zakladniho rozlozeni 
	(argument x) na cislo rozlozeni exponencialniho. Ziskane cislo tvori 
	navratovou hodnotu metody."

	x = 0.0
		ifTrue: [^(1 / 1.1e-37) ln negated / Mu]
		ifFalse: [^x ln negated / Mu]!

setParameter: x 
	"Ulozi do promenne tridy parametr exponencialniho rozlozeni."

	Mu := x! !

!Exponential class methodsFor: 'instance creation'!

mean: p 
	"Pozadavek na vytvoreni noveho cisla z exponencialniho rozlozeni s 
	parametrem p (predstavuje stredni hodnotu)."

	^self parameter: 1.0 / p!

parameter: p 
	"Pozadavek na generovani noveho cisla z exponencialniho rozlozeni s 
	parametrem 1/p (1/p predstavuje stredni hodnotu). Nejdrive je 
	nastavena promenna tridy predstavujici parametr exponencialniho 
	rozlozeni a potom se provadi transformace cisla ze zakladniho do 
	exponencialniho rozlozeni (viz metoda generate: x)."

	p > 0.0
		ifTrue: 
			[self setParameter: p.
			^self next]
		ifFalse: [MySimulation active error: 'Parametr u exponencialniho rozlozeni musi byt vetsi nez 0.']! !

BasicRandom subclass: #Geometric
	instanceVariableNames: ''
	classVariableNames: 'Prob '
	poolDictionaries: ''
	category: 'Probability Distributions'!
Geometric comment:
'Trida ''Geometric'' predstavuje generator geometrickeho rozlozeni s parametrem ''Prob'', ktery predstavuje pravdepodobnost nastoupeni jevu v jednom pokusu.

Promenne tridy:
 Prob <Float> - parametr geometrickeho rozlozeni'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Geometric class
	instanceVariableNames: ''!


!Geometric class methodsFor: 'instance creation'!

mean: m 
	"Pozadavek na generovani noveho cisla z geometrickeho rozlozeni s 
	parametrem m (pravdepodobnost nastoupeni jevu v jednom pokusu)."

	m <= 0
		ifTrue: [MySimulation active error: 'Parametr u geometrickeho rozlozeni musi byt vetsi nez 0']
		ifFalse: 
			[self setParameter: 1 / m.
			^self next]!

parameter: m 
	"Pozadavek na generovani noveho cisla z geometrickeho rozlozeni s 
	parametrem 1/m (1/m predstavuje pravdepodobnost nastoupeni jevu 
	v jednom pokusu)."

	m <= 0
		ifTrue: [MySimulation active error: 'Parametr u geometrickeho rozlozeni musi byt vetsi nez 0.']
		ifFalse: 
			[self setParameter: m.
			^self next]! !

!Geometric class methodsFor: 'private'!

generate: x 
	"Metoda pro transformaci nahodneho cisla ze zakladniho rozlozeni 
	(argument x) na cislo rozlozeni geometrickeho."

	| rand |
	rand := x.
	[rand = 0]
		whileTrue: [rand := BasicRandom next].
	^(rand ln / Prob ln) ceiling!

setParameter: aNumber 
	"Nastavi parametr geometrickeho rozlozeni (aNumber) predstavujici 
	pravdepodobnost nastoupeni jevu v jednom pokusu."

	Prob := 1 - aNumber! !

BasicRandom subclass: #Normal
	instanceVariableNames: ''
	classVariableNames: 'Mu Sigma '
	poolDictionaries: ''
	category: 'Probability Distributions'!
Normal comment:
'Trida ''Normal'' predstavuje generator normalniho rozlozeni s parametry ''Mu'' (stredni hodnota) a ''Sigma'' (smerodatna odchylka). 

Promenne tridy:
  Mu <Float> - parametr normalniho rozlozeni predstavujici jeho stredni hodnotu
  Sigma <Float> - parametr normalniho rozlozeni predstavujici jeho smerodatnou odchylku'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Normal class
	instanceVariableNames: ''!


!Normal class methodsFor: 'private'!

generate: x 
	"Metoda pro generovani nahodnych cisel rozlozeni normalniho s 
	vyuzitim cisel z rovnomerneho rozlozeni v intervalu <-1,1)."

	| v1 v2 s u |
	
	[v1 _ Uniform from: -1.0 to: 1.0.
	v2 _ Uniform from: -1.0 to: 1.0.
	s _ v1 squared + v2 squared.
	s >= 1] whileTrue.
	u _ (-2.0 * s ln / s) sqrt.
	^Mu + (Sigma * v1 * u)!

setMean: m standardDeviation: s 
	"Nastavi promenne tridy podle parametru pozadovaneho rozlozeni."

	Mu := m.
	Sigma := s! !

!Normal class methodsFor: 'instance creation'!

mean: a deviation: b 
	"Pozadavek na generovani noveho cisla z normalniho rozlozeni se 
	stredni hodnotou 'a' a smerodatnou odchylkou 'b'. Nejdrive jsou 
	nastaveny promenne tridy predstavujici parametry normalniho 
	rozlozeni a potom se provadi generovani cisla s vyuzitim generatoru 
	rovnom. rozlozeni."

	b > 0.0
		ifTrue: 
			[self setMean: a standardDeviation: b.
			^self next]
		ifFalse: [MySimulation active error: 'Standardni odchylka musi byt vetsi nez 0']! !